#
# ZooDocument - Zope OpenOffice Document
#
# (c) Copyright 2002 by Philipp "philiKON" von Weitershausen
#                       philikon@philikon.de
#
# Released unter the terms of the Mozilla Public License (MPL) v1.1
#
# Please see README.txt for further information
#

"""
ZooDocument - Zope OpenOffice Document

$Id$
"""

import Globals
from StringIO import StringIO
from AccessControl import ClassSecurityInfo
from OFS.ObjectManager import ObjectManager
from Products.PageTemplates.ZopePageTemplate import ZopePageTemplate
from Products.PageTemplates.PageTemplateFile import PageTemplateFile
from OFS.Image import Image, cookId
from OOoHtml import OOoHtml, proper_extension


VIEW_PERM = "View"
CHANGE_PERM = "Change OpenOffice.org Documents"


class ZooDocument( ObjectManager, ZopePageTemplate ):
    """
    ZooDocument
    """

    meta_type = "OpenOffice.org Document"
    icon = "www/zoodoc.png"
    manage_options = ( {'label':'Document', 'action':'oo_document' },
                       {'label':'Pictures', 'action':'oo_pictures' }, ) + ZopePageTemplate.manage_options

    security = ClassSecurityInfo()

    def __init__( self, id, title, docfile="" ):
        self.id = id
        self.title = title

        if type( docfile ) is not type(""):
            docfile = docfile.read()
            
        self._document = docfile


    def manage_afterAdd( self, item, container ):
        # this has to be done here (not in __init__) because we need acquisition
        # for the patcher
        self.processDocument()


    def _setId( self, id ):
        self.id = id
        # this has to be done after every rename because of the picture links
        self.processDocument()


    security.declarePrivate( "processDocument" )
    def processDocument( self ):
        """
        Process the OpenOffice.org document
        """
        # Run the translation
        converter = OOoHtml( StringIO( self._document ) )
        html = converter.getHtml()

        # add the id in front of all picture links
        html = html.replace( "#Pictures/", self.getId()+"/" )

        # this allows us to install a script or something similar that
        # patches the HTML source before it is written to the Page Template
        try:
            patcher = getattr( self, "zooPatch" )
            html = patcher( html )
        except:
            pass
        
        self.pt_edit( html, "text/html" )

        # extract the metadata
        self._doctype = converter.getDocType()
        self._meta = converter.getMetaData()

        # pictures
        self.manage_delObjects( self.objectIds() )
        for filename, data in converter.getPictures().items():
            image = Image( filename, "", data )
            self._setObject( filename, image )
        
        
    security.declareProtected( VIEW_PERM, "getDocType" )
    def getDocType( self ):
        return self._doctype


    security.declareProtected( VIEW_PERM, "getMetaData" )
    def getMetaData( self ):
        return self._meta


    ## Methods for returning/editing the document ##

    security.declareProtected( VIEW_PERM, "get_size" )
    def get_size( self ):
        return len( self._document )


    ## Zope Management Interface (ZMI) methods ##

    security.declareProtected( CHANGE_PERM, "oo_document" )
    oo_document = PageTemplateFile( "www/oo_document.pt", globals(), __name__="oo_document" )


    security.declareProtected( CHANGE_PERM, "oo_pictures" )
    oo_pictures = manage_main = PageTemplateFile( "www/oo_pictures.pt", globals(), __name__="oo_pictures" )
    

    security.declareProtected( CHANGE_PERM, "oo_download" )
    def oo_download( self, REQUEST, RESPONSE ):
        """
        Download the document for editing
        """
        splitted_filename = self.getId().split(".")
        ext = proper_extension[self.getDocType()]

        if splitted_filename[-1] != ext:
            if len(splitted_filename) > 1:
                splitted_filename[-1] = ext
            else:
                splitted_filename.append( ext )
        filename = ".".join( splitted_filename )
        
        RESPONSE.setHeader( 'Content-type','application/data' )
        RESPONSE.setHeader( 'Content-Disposition', 'inline;filename=%s' % filename )
        return self._document


    security.declareProtected( CHANGE_PERM, "oo_upload" )
    def oo_upload( self, docfile="", REQUEST=None ):
        """
        Replace the contents of the document with the text in file.
        """

        if self.wl_isLocked():
            raise ResourceLockedError, ( 'This document has been locked via WebDAV.' )

        if type( docfile ) is not type(""):
            docfile = docfile.read()
        self._document = docfile
        self.processDocument()

        if REQUEST is not None:
            message = "OpenOffice.org document uploaded."
            return self.oo_document( self, REQUEST, manage_tabs_message=message )


    security.declareProtected( CHANGE_PERM, "oo_download" )
    def oo_convert( self, REQUEST ):
        """
        Convert the document
        """
        self.processDocument()
        if REQUEST is not None:
            message = "OpenOffice.org document converted."
            return self.oo_document( self, REQUEST, manage_tabs_message=message )


    def om_icons( self ):
        """
        Return a list of icon URLs to be displayed by an ObjectManager
        """
        icons = ( {'path': 'misc_/ZooDocument/zoodoc.png',
                   'alt':   self.meta_type,
                   'title': self.meta_type },)

        if not self._v_cooked:
            self._cook()
        if self._v_errors:
            icons = icons + ({'path': 'misc_/PageTemplates/exclamation.gif',
                              'alt': 'Error',
                              'title': 'This template has an error'},)
        return icons

        
    ## Protocol handlers ##

    security.declareProtected( CHANGE_PERM, "PUT" )
    def PUT( self, REQUEST, RESPONSE ):
        """
        Handle HTTP PUT requests.
        """
        self.dav__init( REQUEST, RESPONSE )
        self.dav__simpleifhandler( REQUEST, RESPONSE, refresh=1 )
        body = REQUEST.get( 'BODY', '' )
        self.oo_upload( body )
        RESPONSE.setStatus( 204 )
        return RESPONSE


    security.declareProtected( VIEW_PERM, "manage_FTPget" )
    def manage_FTPget( self ):
        """
        Get source for FTP download
        """
        return self._document



Globals.InitializeClass( ZooDocument )


### object constructors ###

addZooDocumentForm = PageTemplateFile( "www/addZooDoc.pt", globals(), __name__="addZooDocumentform" )


def addZooDocument( self, id="", title="", docfile=None, REQUEST=None ):
    """
    Create a ZooDocument instance to an ObjectManager
    """

    id, title = cookId( id, title, docfile )
    obj_instance = ZooDocument( id, title, docfile )
    self.Destination()._setObject( id, obj_instance )

    if REQUEST is None:
        return

    try:
        url = self.DestinationURL()
    except:
        url = REQUEST['URL1']
    if REQUEST['submit'] == " Add and Edit ":
        url = "%s/%s" % ( url, id )
        REQUEST.RESPONSE.redirect( url + '/oo_document' )
    else:
        REQUEST.RESPONSE.redirect( url + "/manage_main" )

